@@ -10,13 +10,21 @@ def w() -> Wing:
                 xyz_le=np.array([1, 1, 0]),
                 chord=0.5,
                 twist=5,
-                control_surface_is_symmetric=True
+                control_surfaces=[
+                    ControlSurface(
+                        symmetric=True,
+                    )
+                ]
             ),
             WingXSec(
                 xyz_le=np.array([2, 2, 0]),
                 chord=0.5,
                 twist=5,
-                control_surface_is_symmetric=True
+                control_surfaces=[
+                    ControlSurface(
+                        symmetric=True
+                    )
+                ]
             )
         ],
         symmetric=True
