@@ -5,12 +5,15 @@ from aerosandbox.numpy.determine_type import is_casadi_type
 from numpy.linalg import *
 
 
-def inner(x, y):
+def inner(x, y, manual=False):
     """
     Inner product of two arrays.
 
     See syntax here: https://numpy.org/doc/stable/reference/generated/numpy.inner.html
     """
+    if manual:
+        return sum([xi * yi for xi, yi in zip(x, y)])
+
     if not is_casadi_type([x, y], recursive=True):
         return _onp.inner(x, y)
 
@@ -18,12 +21,21 @@ def inner(x, y):
         return _cas.dot(x, y)
 
 
-def outer(x, y):
+def outer(x, y, manual=False):
     """
     Compute the outer product of two vectors.
 
     See syntax here: https://numpy.org/doc/stable/reference/generated/numpy.outer.html
     """
+    if manual:
+        return [
+            [
+                xi * yi
+                for yi in y
+            ]
+            for xi in x
+        ]
+
     if not is_casadi_type([x, y], recursive=True):
         return _onp.outer(x, y)
 
@@ -145,3 +157,53 @@ def norm(x, ord=None, axis=None):
                 print(e)
                 raise ValueError("Couldn't interpret `ord` sensibly! Tried to interpret it as a floating-point order "
                                  "as a last-ditch effort, but that didn't work.")
+
+
+def inv_symmetric_3x3(
+        m11,
+        m22,
+        m33,
+        m12,
+        m23,
+        m13,
+):
+    """
+    Explicitly computes the inverse of a symmetric 3x3 matrix.
+
+    Input matrix (note symmetry):
+
+    [m11, m12, m13]
+    [m12, m22, m23]
+    [m13, m23, m33]
+
+    Output matrix (note symmetry):
+
+    [a11, a12, a13]
+    [a12, a22, a23]
+    [a13, a23, a33]
+
+    From https://math.stackexchange.com/questions/233378/inverse-of-a-3-x-3-covariance-matrix-or-any-positive-definite-pd-matrix
+    """
+    det = (
+            m11 * (m33 * m22 - m23 ** 2) -
+            m12 * (m33 * m12 - m23 * m13) +
+            m13 * (m23 * m12 - m22 * m13)
+    )
+    inv_det = 1 / det
+    a11 = m33 * m22 - m23 ** 2
+    a12 = m13 * m23 - m33 * m12
+    a13 = m12 * m23 - m13 * m22
+
+    a22 = m33 * m11 - m13 ** 2
+    a23 = m12 * m13 - m11 * m23
+
+    a33 = m11 * m22 - m12 ** 2
+
+    a11 = a11 * inv_det
+    a12 = a12 * inv_det
+    a13 = a13 * inv_det
+    a22 = a22 * inv_det
+    a23 = a23 * inv_det
+    a33 = a33 * inv_det
+
+    return a11, a22, a33, a12, a23, a13
